#include <alloc.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <dir.h>
#include <time.h>
#include "announce.h"
#include "dx.h"
#include "tools.h"
#include "cshow.h"
#include "node.h"
#include "define.h"
#include "buffers.h"
#include "message.h"
#include "streams.h"
#include "commands.h"
#include "protocol.h"
#include "database.h"
#include "language.h"
#include "hops.h"
#include "motd.h"
#include "time.h"
#include "dxcc.h"
#include "switch.h"
extern "C" {
#include "memory.h"
}
#include "system.h"
#include "wwv.h"
#include "filter.h"
#include "users.h"
#include "locator.h"

#define SPECIAL_DX_


/*------------------------------------------------------------------------
  ------ Rediriger les commandes SHOW ------------------------------------
  ------------------------------------------------------------------------
  La fonction return TRUE si la commande a ete executee                   */
#pragma argsused
int CSHOW_execute(int StreamNum, char * pCmd_p, int nParams, char **pArg, char **pStrArg)
{
  int Length;			/*Longueur de la commande*/

  if( *pArg[0] == SNULL )
    return FALSE;

  /*Longueur de la commande*/
  Length = strlen (pArg[0]);

  if( COMMANDS_check(pArg[0], "announce", Length, 1) || ! strncmp(pArg[0], "a/", 2) )
    return CSHOW_announce(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "beam", Length, 1) )
    return CSHOW_beam(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "cluster", Length, 2) )
    return CSHOW_cluster(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "configuration", Length, 1) )
    return CSHOW_configuration(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "c/n", Length, 3) )
    return CSHOW_nodeConfiguration(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "date", Length, 4) )
    return CSHOW_date(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "memleft", Length, 3) )
    return CSHOW_memleft(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "database", Length, 4) )
    return CSHOW_database(StreamNum, nParams, pArg, pStrArg);

  if( ! strcmp(pArg[0], "dx") || ! strncmp(pArg[0], "dx/", 3) )
    return CSHOW_dx(StreamNum, nParams, pArg, pStrArg);

  if( ! strcmp(pArg[0], "fc") )
    return CSHOW_fileClosed(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "hops", Length, 3) )
    return CSHOW_hops(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "language", Length, 3) )
    return CSHOW_language(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "motd", Length, 3) )
    return CSHOW_motd(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "prefix", Length, 1) )
    return CSHOW_prefix(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "station", Length, 2) )
    return CSHOW_station(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "time", Length, 1) )
    return CSHOW_time(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "tnc_status", Length, 3) )
    return CSHOW_tnc_status(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "users", Length, 1) )
    return CSHOW_users(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "usercmd", Length, 5) )
    return CSHOW_usercmd(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "version", Length, 3) )
    return CSHOW_version(StreamNum, nParams, pArg, pStrArg);

  if( COMMANDS_check(pArg[0], "wwfilter", Length, 3) )
    return CSHOW_wwfilter(StreamNum, nParams, pArg, pStrArg);

  if( ! strcmp(pArg[0], "wwv") || ! strncmp(pArg[0], "wwv/", 4) )
    return CSHOW_wwv(StreamNum, nParams, pArg, pStrArg);

  /*Interrogation d'une base de donnees locale ?*/
  if( DATABASE_local(StreamNum, pArg[0], pArg[1], DB_LOCAL) )
  {
    /*Commande database OK*/
    Prompt;
    return TRUE;
  }

  return FALSE;		/*La commande n'a pas ete trouvee*/
}

/*------------------------------------------------------------------------
  ------ Annonce ---------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_announce(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  int	iNbr = 5;       	/*Nombre d'annonce a afficher*/
  int	Done = FALSE;		/*TRUE si une annonce a ete trouvee*/
  int	index;
  char 	sUserCall[16];		/*Utilise par ANNOUNCE_get*/
  char	sNodeCall[16];		/*idem*/
  char 	sString[256];           /*idem*/
  char	sToWho[32];             /*idem*/
  char	cFlag;                  /*idem*/
  char 	sDate[16];
  char 	sTime[16];
  time_t tm;

  unsigned long 	lDateTime;

  /*Nombre d'annonces  renvoyer*/
  if( pArg[0][1] == '/' )
  {
    if( pArg[0][2] != SNULL )	/*Proteger le atoi*/
    {
      iNbr = atoi (pArg[0] + 2);

      if( iNbr <= 0 )
	iNbr = 5;
    }
  }/*End IF*/

  /* Pour une recherche non "case sensitive" */
  if( *pArg[1] != SNULL )
	strlwr(pArg[1]);

  /*Lire les iNbr annonces*/
  tm = time(NULL);
  for(index = 1; index <= iNbr; index++)
  {
    if( ! ANNOUNCE_get(index, sNodeCall, sToWho, sUserCall, &cFlag, sString, &lDateTime) )
	break;	/*La fin de la liste a t atteinte*/

    /* Si des conditions ont t donnes est qu'elles sont remplies - si pas de condx. */
    strlwr(sString);
    if( *pArg[1] == SNULL || strstr(sString, pArg[1]) )
    {
      /*Convertir la date*/
      TOOLS_dateTime_Long2Str(sDate, sTime, lDateTime);

      /*Adresser l'annonce  l'utilisateur*/
      BUFFERS_printBuff(StreamNum, OUT, "%-8s de %-9s %10s <%4sZ>:\n%s\n",
			sToWho, sUserCall, sDate, sTime, sString);

      Done = TRUE;
    }
    else
    {
      /*Si le programme arrive ici, c'est qu'une condition de recherche
	n'a pas t trouve. Ajouter 1  iNbr */
      iNbr++;

      /* Timeout de 30 secopndes pour la recherche */
      if( time(NULL) > tm + 30 )
	break;
    }/*End IF*/
  }/*End FOR*/

  /*Afficher un message si aucune correspodance n'a t trouvee*/
  if( ! Done )
    MSG_send(StreamNum, MSG7);

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/Beam ---------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_beam(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
	char szToLocator[8];
	char szFromLocator[8];
	extern tUser   *pUser;

	LOCATOR_COORDINATES from;
	LOCATOR_COORDINATES to;
	
	/* Un argument pour cette fonction */
	if( nParams != 2 )
	{
		MSG_send(StreamNum, MSG58);
		Prompt;
		return TRUE;
	}

	TOOLS_maxLength(pArg[1], 6);
	strcpy(szToLocator, pArg[1]);
	strupr(szToLocator);

	/* Le locator est il valide ? */
	if( ! LOCATOR_isValidWW(szToLocator) )
	{
		/* Locator incorrect */
		MSG_send(StreamNum, MSG18);
		Prompt;
		return TRUE;
	}

	/* Recuperer mon locator en memoire */
	USERS_getRecord(StreamNum);

	if( ! LOCATOR_isValidWW(pUser->sLocator) )
	{
		/* locator pas valide */
		MSG_send(StreamNum, MSG71);
		Prompt;
		return TRUE;
	}

	strcpy(szFromLocator, pUser->sLocator);

	/* Calcul de la distance et de l'azimut */
	LOCATOR_locator2coordinates(szFromLocator, &from);
	LOCATOR_locator2coordinates(szToLocator, &to);

	/* Trois appels (il faut s'assurer que azimut soit appele apres
	   distance */
	BUFFERS_printBuff(StreamNum, OUT, "%s -> %s : ",
			  pUser->sLocator,
			  szToLocator);
	BUFFERS_printBuff(StreamNum, OUT, "%d km, ",
			  LOCATOR_distance(from, to));
	BUFFERS_printBuff(StreamNum, OUT, "%d\n",
			  LOCATOR_azimut(from, to));

	Prompt;
	
	return TRUE;
}


/*------------------------------------------------------------------------
  ------ SH/CLuster ------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_cluster(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  extern time_t	MAIN_timeStart;
  int	iNodes,
	iLocalUsers,
	iTotalUsers,
	iMaxUsers;
  int   iDays,
	iHours,
	iMin;

  /*Pas d'arguments pour cette fonction*/
  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);
    Prompt;
    return TRUE;
  }

  /*Quelle est la config ?*/
  NODE_getConfig(iNodes, iLocalUsers, iTotalUsers, iMaxUsers);
  TOOLS_diffTime(MAIN_timeStart, time(NULL), &iDays, &iHours, &iMin);

  BUFFERS_printBuff(StreamNum, OUT,
    "Cluster: %d nodes, %d local / %d total users - Max users %d Uptime %dd %02d:%02d\n",
    iNodes, iLocalUsers, iTotalUsers, iMaxUsers,
    iDays, iHours, iMin);


  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/Configuration ------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_configuration(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  extern int 	LastStream;	/*switch.cpp*/
  unsigned char cFlag;
  char		sNodeCall[16];
  char		sUserCall[16];
  char		sCallSign[16];
  char		sFilter[16];
  int		iFilterLength;
  int		iNb;	        /*Nombre d'utilisateurs affiches*/
  int		iStream;	/*Numero du stream*/
  int		iNodePos;	/*Position du node dans le stream*/
  int		iUserPos;	/*Position des utilisateurs*/

  /*Pas d'arguments pour cette fonction*/
  if( nParams > 2 )
  {
    MSG_send(StreamNum, MSG58);
    Prompt;
    return TRUE;
  }

  /*L'utilisateur a t-il specifie un filtre ?*/
  if( nParams == 2 )
  {
    TOOLS_maxLength(pArg[1], 15);
    strcpy(sFilter, pArg[1]);
    strupr(sFilter);
    iFilterLength = strlen (sFilter);
  }
  else
  {
    *sFilter = SNULL;
    iFilterLength = 0;
  }

  /*Afficher l'entete*/
  MSG_send(StreamNum, MSG6);	/*Cluster configuration*/

  /*------ Parcourir la table ------*/
  for(iStream = 0; iStream <= LastStream; iStream++)
  {
    for(iNodePos = 0; iNodePos < NODE_MAXANODE; iNodePos++ )
    {
      /*Indicatif du node sur le stream*/
      NODE_getNodeCall(iStream, iNodePos, sNodeCall);

      /*Pas d'indicatif -> iNodePos suivant*/
      if( ! *sNodeCall )
	continue;

      /*Ne rentre pas dans le filtre -> iNodePos suivant*/
      if( *sFilter && strncmp(sNodeCall, sFilter, iFilterLength) )
	continue;

      BUFFERS_printBuff(StreamNum, OUT, "%-14s", sNodeCall);

      iNb = 0;

      /*Liste des utilisateurs*/
      for(iUserPos = 1; iUserPos <= 64; iUserPos++)
      {
	/*Indicatif de l'utilisateur*/
	NODE_getUserCall(iStream, iNodePos, iUserPos, sUserCall);

	/*Pas d'utilisateur -> iUserPos suivant*/
	if( ! *sUserCall )
	  continue;

	/*Faut-il passer a la ligne ?*/
	if( iNb++ == 5 )
	{
	  iNb = 1;
	  BUFFERS_addBuff(StreamNum, "\n              ", OUT);
	}

	/*Quelle est la configuration de l'utilisateur ?*/
	cFlag =  NODE_getUserFlag(iStream, iNodePos, iUserPos);
	if( cFlag & PROTOCOL_USER_HERE )
	  strcpy (sCallSign, sUserCall);		/*SET/HERE*/
	else                                            /*SET/NOHERE*/
	  sprintf(sCallSign, "(%s)", sUserCall);

	if( cFlag & PROTOCOL_USER_CONF )		/*Conference ?*/
	  strcat(sCallSign, "+");

	BUFFERS_printBuff(StreamNum, OUT, "%-12s", sCallSign);

      }/*End iUserPos FOR*/

      if( iNb == 0 ) /*Pas d'utilisateurs connectes*/
      {
	int iUserCount = NODE_getUserCount(iStream, sNodeCall);

	if( iUserCount != 0 )
	  BUFFERS_printBuff(StreamNum, OUT, "(%d users)", iUserCount);
      }/*End IF*/

      BUFFERS_addBuff(StreamNum, "\n", OUT);

    }/*End iNodePos FOR*/
  }/*End iStream FOR*/

#ifdef SPECIAL_DX_
  {
    extern char	STREAMS_level[MAX_STREAMS];
    extern char	STREAMS_callsign[MAX_STREAMS][10];

    if( STREAMS_level[StreamNum] & LEVEL_sysopD )
    {
      for(iStream = 1; iStream <= LastStream; iStream++)
      {
	if( HOPS_fix(iStream, PCAS_USER, IN, 100) )
	  BUFFERS_printBuff(StreamNum, OUT, "<%s>\n",
			    STREAMS_callsign[iStream]);
      }
    }
  }/*End IF*/
#endif

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/DATE ---------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_date(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  extern long   PARAMS_lTz;
  time_t t;
  struct tm tDate;
  char   szBuffer[256];
  extern char	STREAMS_level[MAX_STREAMS];

  /* RX only ? */
  if( STREAMS_level[StreamNum] & LEVEL_user_RXonly )
	return FALSE;

  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Pas d'argument pour cette cmd*/
    Prompt;
    return TRUE;
  }

  t     = time(NULL) - PARAMS_lTz;
  tDate = *localtime(&t);
  strftime(szBuffer, 255, "%a %b %d, %Y", &tDate);
  BUFFERS_printBuff(StreamNum, OUT, "Current date : %s (%04d.%02d.%02d)\n",
		    szBuffer,
		    tDate.tm_year + 1900,
		    tDate.tm_mon + 1,
		    tDate.tm_mday);

  Prompt;
  return TRUE;
}


/*------------------------------------------------------------------------
  ------ SH/MEMleft ------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_memleft(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
#ifdef DOS
  unsigned int 	iHandler;
  unsigned long lConventional = 0L;
  unsigned long lExtended     = 0L;
  unsigned long lExpanded     = 0L;
  unsigned long lVirtual      = 0L;
  char          *pMemoryType;
  unsigned int	iCounter      = 0;


  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Pas d'argument pour cette cmd*/
    Prompt;
    return TRUE;
  }

  BUFFERS_printBuff(StreamNum, OUT, "%ld bytes free in far heap.\n", farcoreleft() );

  /*Scanner la memoire utilise par le memory manager*/
  for(iHandler = 0; iHandler < 32767; iHandler++)
  {
    pMemoryType = mem_type( iHandler );

    if( *pMemoryType != 'N' )	/*Not allocated*/
    {
      iCounter = 0;

      if( *pMemoryType == 'C' )
	lConventional += (unsigned long) get_memsize(iHandler);
      else if( *(pMemoryType + 2) == 't' )
	lExtended     += (unsigned long) get_memsize(iHandler);
      else if( *(pMemoryType + 2) == 'p' )
	lExpanded     += (unsigned long) get_memsize(iHandler);
      else if( *pMemoryType == 'V' )
	lVirtual      += (unsigned long) get_memsize(iHandler);
     }
     else
     {
       /*Au dela de 500 ihandler vide consecutif, on suppose
	 qu'il n'y en aura pas d'autres utilises. */
       if( iCounter++ > 500 )
	 break;
     }
   }/*End FOR*/

   BUFFERS_printBuff(StreamNum, OUT, "The memory manager uses : %4ld Kb of Conventional memory.\n",   lConventional / 1024L);
   BUFFERS_printBuff(StreamNum, OUT, "                          %4ld Kb of Extented (XMS) memory.\n", lExtended / 1024L);
   BUFFERS_printBuff(StreamNum, OUT, "                          %4ld Kb of Expanded (EMS) memory.\n", lExpanded / 1024L);
   BUFFERS_printBuff(StreamNum, OUT, "                          %4ld Kb of Virtual memory.\n",        lVirtual / 1024L);

 #ifdef DEBUG_LOG_
   BUFFERS_printBuff(StreamNum, OUT, "%u bytes available in stack.\n", stackavail());
   if( SET_check(PARAMS_logs, LOG_MEM_STR) )
     LOG_write(LOG_MEM_FILE, LOG_DATE_YES, "\nfar   %6ld bytes\nCONV  %6ld Kb\nXMS   %6ld Kb\nEMS   %6ld Kb\nVIRT  %6ld Kb\nSTACK %6u bytes\n\n",
	       farcoreleft(), lConventional / 1024L, lExtended / 1024L,
	       lExpanded / 1024L, lVirtual / 1024L, stackavail() );
 #endif
#else
  BUFFERS_addBuff(StreamNum, "This command is not implemented for the linux version.\n", OUT);
  /*BUFFERS_printBuff(StreamNum, OUT, "%ld bytes free.\n", farcoreleft() );*/
#endif

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/database -----------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_database(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Pas d'argument pour cette cmd*/
    Prompt;
    return TRUE;
  }

  DATABASE_showList(StreamNum);

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/DX -----------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_dx(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  int	 iBand = -1;  		/*Bande sur laquelle effectuer la recherche*/
  int	 iNbr  = 5;
  char * pPtr;
  int	 index;
  int 	 Done;
  time_t tm;
  DXSPOT DxSpot;

  extern int	PARAMS_iSetDxSsid;

  /*Nombre de DX a renvoyer*/
  if( pArg[0][2] == '/' )
  {
	if( pArg[0][3] != SNULL )	/*Proteger le atoi*/
	{
		iNbr = atoi (pArg[0] + 3);

		if( iNbr <= 0 )
			iNbr = 5;
	}
  }/*End IF*/

  /* S'il y a des arguments */
  if( nParams != 1 )
  {
	/*Recherche en majuscule uniquement*/
	strupr(pArg[1]);
	strupr(pArg[2]);

	/*Recherche sur une bande particuliere */
	pPtr = pArg[1];
	Done = TRUE;

	while( *pPtr )
	{
		if( ! isdigit(*pPtr++) )
			Done = FALSE;	/*Autre chose de des chiffres ont ete trouve*/
	}/*End WHILE*/

	iBand = atoi(pArg[1]);
	if( iBand == 0 || Done == FALSE )
		iBand = -1;
	else
		strcpy(pArg[1], "###");	/*Copier qq chose que le test suivant ne pourra confondre avec un indicatif*/

	/*Remplacer, le cas echeant, des frequences par des bandes*/
	switch( iBand )
	{
	case 144:
		iBand = 2;
		break;
	case 50:
		iBand = 6;
		break;
	case 432:
		iBand = 70;
		break;
	case 1296:
		iBand = 23;
		break;
	}/*End SWITCH*/
   }/*End IF*/

   /*Done est utilise pour s'assurer qu'une correspondance a ete trouve
     lors de la recherche*/
   Done = FALSE;

   /*Parcourir le fichier*/
   tm = time(NULL);
   for(index = 1; index <= iNbr; index++)
   {
	if( ! DX_get(index, &DxSpot) )
		break;	/*La fin de la liste  t atteinte*/

	/*Si des conditions ont t donnes et qu'elles sont remplies - si
	  pas de condx, afficher */
	if( ! *pArg[1] || 				    		/*S'il n'y a aucune conditions*/
	   (! *pArg[2] && strstr(DxSpot.szDxCall, pArg[1]))   || 	/*Recherche sur une partie de l'indicatif*/
	   (  *pArg[2] && strstr(DxSpot.szDxCall, pArg[2] )   &&   /*Recherche sur une partie de l'indicatif*/
	      iBand == DX_checkFrequency(DxSpot.lFrequency) ) ||
	   (  iBand == DX_checkFrequency(DxSpot.lFrequency)   && 	/*Recherche sur la bande*/
	    ! *pArg[2] ))
	{
		char szLogger[16];

		/* Faut il enlever le SSID du logger ? */
		strcpy(szLogger, DxSpot.szLogger);
		if( PARAMS_iSetDxSsid == FALSE )
			TOOLS_removeSsid(szLogger);
		strcat(szLogger, ">");

		TOOLS_maxLength(DxSpot.szComments, 28);	/*Ne pas dpasser la limite sur l'cran*/
		TOOLS_maxLength(DxSpot.szDxCall, 13);	/*idem*/
		BUFFERS_printBuff(StreamNum, OUT, "%8ld.%ld %-13s %11s %4sZ %-28s<%-6s\n",
			 DxSpot.lFrequency/10, DxSpot.lFrequency%10,
			 DxSpot.szDxCall, DxSpot.szDate, DxSpot.szTime,
			 DxSpot.szComments, szLogger);
		Done = TRUE;
	}
	else
	{
		/*Si le programme arrive ici, c'est qu'une condition de recherche
		  n'a pas t trouve. Ajouter 1  iNbr*/
		iNbr++;

		/* Timeout de 30 secopndes pour la recherche */
		if( time(NULL) > tm + 30 )
			break;
	}
   }/*End FOR*/

   /* Afficher un message si aucune correspodance n'a t trouvee */
   if( ! Done )
	MSG_send(StreamNum, MSG7);

   Prompt;
   return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/FC -----------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_fileClosed(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  static int iFileClosed = 0;	/*Nombre de fichier qui n'ont pas ete ouvert*/

  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Pas d'argument pour cette cmd*/
    Prompt;
    return TRUE;
  }

  iFileClosed += fcloseall();

  BUFFERS_printBuff(StreamNum, OUT, "%d files have been abnormaly closed.\n", iFileClosed);

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/HOPs ---------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_hops(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  int	iPc16,
	iPc19,
	iPc50;
  int	iDone = FALSE;
  int	index;
  char	sBuffer[256];

  /*Si l'utilisateur a prcis un indicatif ...*/
  if( nParams > 1 )
  {
    strupr (pArg[1]); 	/*Convertir en majuscule pour la recherche*/

    /*Verifier que l'indicatif est valide*/
    if( ! TOOLS_isCall (pArg[1]) )
    {
      MSG_send(StreamNum, MSG18);
      Prompt;
      return TRUE;
    }

    if( HOPS_getHops(pArg[1], 0, iPc16, iPc19, iPc50) )
    {
      MSG_send(StreamNum, MSG30);
      COMMANDS_sendHops(StreamNum, pArg[1], iPc16, iPc19, iPc50, TRUE);  /*Envoyer le tableau  l'utilisateur*/
    }
    else
      MSG_send(StreamNum, MSG31);
  }
  else
  {
    int nCR = FALSE;

    /*Pas d'indicatif donn, parcourir la table*/
    for(index = 0; index < 5000; index++) /*5000 tant dj bcp !!*/
    {
      *sBuffer = SNULL;

      if( ! HOPS_getHops(sBuffer, index, iPc16, iPc19, iPc50) )
	break;

      if( iDone == FALSE )
      {
	char	szBuffer[256];

	/* Afficher l'entete */
	MSG_get(StreamNum, MSG30, szBuffer);
	TOOLS_removeNR(szBuffer);
	BUFFERS_printBuff(StreamNum, OUT, "%s           %s\n",
			  szBuffer, szBuffer);
	iDone = TRUE;
      }

      /*Envoyer le tableau  l'utilisateur*/
      COMMANDS_sendHops(StreamNum, sBuffer, iPc16, iPc19, iPc50, nCR);

      /* Pour l'affichage sur deux colonnes */
      if( nCR == FALSE )
      {
	BUFFERS_printBuff(StreamNum, OUT, "            ");
	nCR = TRUE;
      }
      else
	nCR = FALSE;
    }/*End FOR*/

    if( nCR == TRUE )
	BUFFERS_addBuff(StreamNum, "\n", OUT);

  }/*End IF*/

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/LANguage -----------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_language(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  int 	index;
  char	sBuffer[256];

  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Nbr d'arguments incorrect*/
    Prompt;
    return TRUE;
  }

  /*Afficher l'entete*/
  MSG_send(StreamNum, MSG9);

  /*Balayer la liste des langues disponibles*/
  for(index = 0; index < MAX_LANG; index ++)
  {
    extern char	STREAMS_lang[MAX_STREAMS];
    if( LANG_isDefined(index) )
      BUFFERS_printBuff(StreamNum, OUT, "%c%d : %s",
			(STREAMS_lang[StreamNum] == index ? '*' : ' '),
			index + 1,
			LANG_getMessage(index, LANGUAGE, sBuffer));
  }/*End FOR*/

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/MOTD ---------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_motd(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  extern char	STREAMS_lang[MAX_STREAMS];
  extern char	STREAMS_level[MAX_STREAMS];

  /*Niveau d'utilisateur*/
  if( ! (STREAMS_level[StreamNum] & LEVEL_sysopV) )
    return FALSE;

  if( nParams > 2 )
  {
    MSG_send(StreamNum, MSG58);		/*Nbr d'arguments incorrect*/
    Prompt;
    return TRUE;
  }

  if( nParams == 1 ) 	/*MOTD par defaut, si aucune valeur n'est precisee*/
    MOTD_send(StreamNum, STREAMS_lang[StreamNum]);
  else
  {
    int iNb = atoi (pArg[1]);

    if( iNb < 1 || iNb > MAX_LANG )
      MSG_send(StreamNum, MSG18);
    else
      MOTD_send(StreamNum, iNb - 1);
  }/*End IF*/

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/N/C ----------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_nodeConfiguration(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  extern int 	FirstStream;	/*switch.cpp*/
  extern int 	LastStream;	/*switch.cpp*/
  char		szNodeCall[16];
  char		szFilter[16];
  int		nFilterLength;
  int		nNb;	        /*Nombre d'utilisateurs affiches*/
  int		nStream;	/*Numero du stream*/
  int		nNodePos;	/*Position du node dans le stream*/

  /*Pas d'arguments pour cette fonction*/
  if( nParams > 2 )
  {
    MSG_send(StreamNum, MSG58);
    Prompt;
    return TRUE;
  }

  /*L'utilisateur a t-il specifie un filtre ?*/
  if( nParams == 2 )
  {
    TOOLS_maxLength(pArg[1], 15);
    strcpy(szFilter, pArg[1]);
    strupr(szFilter);
    nFilterLength = strlen (szFilter);
  }
  else
  {
    *szFilter = SNULL;
    nFilterLength = 0;
  }

  /*Afficher l'entete*/
//  MSG_send(StreamNum, MSG6);	/*Cluster configuration*/


  /*------ Parcourir la table ------*/
  for(nStream = FirstStream; nStream <= LastStream; nStream++)
  {
	/* Indicatif du node sur le stream */
	NODE_getNodeCall(nStream, 0, szNodeCall);

	/* Pas d'indicatif -> iNodePos suivant */
	if( ! *szNodeCall )
		continue;

	/* Ne rentre pas dans le filtre -> iNodePos suivant */
	if( *szFilter && strncmp(szNodeCall, szFilter, nFilterLength) )
		continue;

	BUFFERS_printBuff(StreamNum, OUT, "%-14s", szNodeCall);

	nNb = 0;

	for(nNodePos = 1; nNodePos < NODE_MAXANODE; nNodePos++ )
	{
		/* Indicatif du node sur le stream */
		NODE_getNodeCall(nStream, nNodePos, szNodeCall);

		/* Pas d'indicatif -> iNodePos suivant */
		if( ! *szNodeCall )
			continue;

		/* Ne rentre pas dans le filtre -> iNodePos suivant */
		if( *szFilter && strncmp(szNodeCall, szFilter, nFilterLength) )
			continue;

		/* Faut-il passer a la ligne ? */
		if( nNb++ == 5 )
		{
			nNb = 1;
			BUFFERS_addBuff(StreamNum, "\n              ", OUT);
		}

		BUFFERS_printBuff(StreamNum, OUT, "%-12s", szNodeCall);
	}

	BUFFERS_addBuff(StreamNum, "\n", OUT);
  }

  Prompt;
  return TRUE;
}


/*------------------------------------------------------------------------
  ------ SH/PREFIX -------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_prefix(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  if( nParams != 2 )
  {
    MSG_send(StreamNum, MSG58);
    Prompt;
    return TRUE;
  }

  DXCC_search(StreamNum, pArg[1]);
  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/STATION ------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_station(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
	char szStation[16];
	char szDate[16];
	char szTime[16];
	char szLanguage[256];
	extern char	STREAMS_level[MAX_STREAMS];
	tUser  user;

	/* RX only ? */
	if( STREAMS_level[StreamNum] & LEVEL_user_RXonly )
		return FALSE;

	if( nParams > 2 )
	{
		MSG_send(StreamNum, MSG58);
		Prompt;
		return TRUE;
	}

	if( nParams == 1 )
	{
		extern char	STREAMS_callsign[MAX_STREAMS][10];

		strcpy(szStation, STREAMS_callsign[StreamNum]);
	}
	else
	{
		TOOLS_maxLength(pArg[1], 9);
		strcpy(szStation, pArg[1]);
	}

	/* Mettre en forme l'indicatif */
	TOOLS_removeSsid(szStation);
	strupr(szStation);

	/* Rechercher */
	if( ! USERS_getConfig(szStation, &user) )
	{
		/* utilisateur inconnu */
		MSG_send(StreamNum, MSG31);
		Prompt;
		return TRUE;
	}

	/* Date et heure de la derniere connexion */
	TOOLS_dateTime_Long2Str(szDate, szTime, user.lLastConnect);
	LANG_getMessage(user.cLanguage, LANGUAGE, szLanguage);
	TOOLS_removeNR(szLanguage);

	BUFFERS_printBuff(StreamNum, OUT, "User          : %s\n", user.sCallSign);
	BUFFERS_printBuff(StreamNum, OUT, "Name          : %s\n", user.sName);
	BUFFERS_printBuff(StreamNum, OUT, "QTH           : %s\n", user.sQth);
	BUFFERS_printBuff(StreamNum, OUT, "Locator       : %s\n", user.sLocator);
	BUFFERS_printBuff(StreamNum, OUT, "Home node     : %s\n", user.sHomeNode);
	BUFFERS_printBuff(StreamNum, OUT, "Last connect  : %s %sZ\n", szDate, szTime);
	BUFFERS_printBuff(StreamNum, OUT, "Last msg read : %ld\n", user.lLastMsg);
	BUFFERS_printBuff(StreamNum, OUT, "Connections   : %ld\n", user.lLogins);
	BUFFERS_printBuff(StreamNum, OUT, "Language      : %s (#%d)\n", szLanguage, user.cLanguage);


	Prompt;
	return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/TIME ---------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_time(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  extern long   PARAMS_lTz;
  time_t t;
  struct tm tTime;
  char   szBuffer[256];
  extern char	STREAMS_level[MAX_STREAMS];

  /* RX only ? */
  if( STREAMS_level[StreamNum] & LEVEL_user_RXonly )
	return FALSE;

  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Pas d'argument pour cette cmd*/
    Prompt;
    return TRUE;
  }

  t     = time(NULL) - PARAMS_lTz;
  tTime = *localtime(&t);
  strftime(szBuffer, 255, "%X", &tTime);
  BUFFERS_printBuff(StreamNum, OUT, "Current time : %s\n", szBuffer);

  Prompt;
  return TRUE;
}


/*------------------------------------------------------------------------
  ------ SH/TNC_STATUS ---------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_tnc_status(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
	extern char	STREAMS_level[MAX_STREAMS];
	extern char	STREAMS_callsign[MAX_STREAMS][10];
#ifdef LINUX
	extern int	SWITCH_nSocket;
#else
	extern char	TERM_myCall[11];  /*L'indicatif du terminal*/
#endif
	int	index;
	char	szLevel[64];
	char *  pszConnectedCall;
	int	nCount = 0;


	/*Niveau d'utilisateur*/
	if( ! (STREAMS_level[StreamNum] & LEVEL_sysopV) )
		return FALSE;

	if( nParams != 1 )
	{
		MSG_send(StreamNum, MSG58);
		Prompt;
		return TRUE;
	}

#ifndef LINUX
	if( SWITCH_isPcFlexNet() )
		BUFFERS_printBuff(StreamNum, OUT, "PC/FlexNet\n");
	else
		BUFFERS_printBuff(StreamNum, OUT, "G8BPQ switch - %d buffer(s) left\n", SWITCH_getBufferLeft());
#else
	BUFFERS_printBuff(StreamNum, OUT, "Linux kernel\n");
#endif

	for(index = 1; index <= 64; index++)
	{
		if( STREAMS_level[index] == 0 )
			continue;

		nCount++;
		pszConnectedCall = STREAMS_callsign[index];

		if( STREAMS_level[index] & LEVEL_cluster )
			strcpy(szLevel, "cluster");
		else if( STREAMS_level[index] & LEVEL_sysopV )
			strcpy(szLevel, "sysop");
		else if( STREAMS_level[index] & LEVEL_linkSetup )
			strcpy(szLevel, "link setup");
#ifndef LINUX
		else if( STREAMS_level[index] & LEVEL_term )
		{
			strcpy(szLevel, "terminal");
			pszConnectedCall = TERM_myCall;
		}
#endif
		else
			strcpy(szLevel, "user");

		BUFFERS_printBuff(StreamNum, OUT, "channel %2d : %-10s (%s)\n",
						  index,
						  pszConnectedCall,
						  szLevel);
	}

	BUFFERS_printBuff(StreamNum, OUT, "%d port(s) in use\n", nCount);
#ifdef LINUX
	BUFFERS_printBuff(StreamNum, OUT, "%d open socket(s)\n", SWITCH_nSocket);
#endif

	Prompt;
	return TRUE;
}


/*------------------------------------------------------------------------
  ------ SH/USERS --------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_users(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  unsigned char cFlag;
  char		sNodeCall[16];
  char		sUserCall[16];
  char		sCallSign[16];
  int		iNb = 0;
  int		iStreamPos;

  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Nbr d'arguments incorrect*/
    Prompt;
    return TRUE;
  }

  /*Afficher l'entete*/
  MSG_send(StreamNum, MSG40);	/*Cluster configuration*/

  iNb = 0;

  /*Indicatif du cluster*/
  NODE_getNodeCall(0, 0, sNodeCall);
  if( *sNodeCall )	/*S'il y a un indicatif, l'afficher*/
  {
    BUFFERS_printBuff(StreamNum, OUT, "%-14s", sNodeCall);

    /*ICI Afficher la liste des utilisateurs*/
    for(iStreamPos = 1; iStreamPos <= 64; iStreamPos++)
    {
      NODE_getUserCall(0, 0, iStreamPos, sUserCall);
      if( *sUserCall )	/*S'il y a un indicatif, l'afficher*/
      {
	if( iNb++ == 5 )/*Passer  la ligne ?*/
	{
	  iNb = 1;
	  BUFFERS_addBuff(StreamNum, "\n              ", OUT);
	}

	/*Quelle est la configuration de l'utilisateur ?*/
	cFlag =  NODE_getUserFlag(0, 0, iStreamPos);

	if( cFlag & PROTOCOL_USER_HERE )
	  strcpy(sCallSign, sUserCall);                 /*SET/HERE*/
	else
	  sprintf(sCallSign, "(%s)", sUserCall);        /*SET/NOHERE*/

	if( cFlag & PROTOCOL_USER_CONF )
	  strcat(sCallSign, "+");

	BUFFERS_printBuff(StreamNum, OUT, "%-12s", sCallSign);
      }/*End IF*/
    }/*End FOR*/

    BUFFERS_addBuff(StreamNum, "\n", OUT);

  }/*End IF*/

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/USERCMD ------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_usercmd(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  extern char	STREAMS_callsign[MAX_STREAMS][10];
  char	sFileName[MAXPATH];
  char	sUserCall[16];

  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Nbr d'arguments incorrect*/
    Prompt;
    return TRUE;
  }


  /*Indicatif de l'utilisateur effectuant la requete*/
  strcpy(sUserCall, STREAMS_callsign[StreamNum]);
  TOOLS_removeSsid(sUserCall);
  strlwr(sUserCall);

  sprintf(sFileName, "%s%s.cmd", USERCMD_PATH, sUserCall);

  if( ! SYSTEM_showFile(StreamNum, sFileName) )
  {
    /*S'il n'y a pas de config pour l'utilisateur, renvoyer la
      config par defaut*/
    sprintf(sFileName, "%sdefault.cmd", USERCMD_PATH);
    SYSTEM_showFile(StreamNum, sFileName);
  }

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/VERSION ------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_version(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  char	sNodeCall[16];
  int	iStreamOfNode;
  int	iNodePos;

  if( nParams > 2 )
  {
    MSG_send(StreamNum, MSG58);		/*Nbr d'arguments incorrect*/
    Prompt;
    return TRUE;
  }

  if( nParams == 1 )
  {
     /*Indicatif du cluster non specifie - utiliser celui du serveur*/
     NODE_getNodeCall(0, 0, sNodeCall);
  }
  else
  {
    TOOLS_maxLength (pArg[1], 9);
    strcpy(sNodeCall, strupr (pArg[1]));
  }

  /*Rechercher le cluster en memoire*/
  if( NODE_isNodeConnected(sNodeCall, &iStreamOfNode, &iNodePos) )
  {
    char	sVersion[16];

    strcpy(pArg[2], NODE_getNodeVersion(iStreamOfNode, iNodePos, sVersion));
    MSG_send(StreamNum, MSG44);	/*Transmettre le numero de version*/
  }
  else
  {
    MSG_send(StreamNum, MSG19);	/*Pas trouve*/
  }/*End IF*/

  Prompt;
  return TRUE;
}

/*------------------------------------------------------------------------
  ------ SH/WWFILTER -----------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_wwfilter(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  unsigned int nFlags;
  char         szBuffer[256];
  extern char *MSG_pParams[10];

  /* Cette commande n'existe que si SET/PFILTER est ON */
  if( ! FILTER_isEnabled() )
	return FALSE;

  /* Ne pas autoriser les remote commands */
  if( StreamNum >= RCMD_VIRTSTREAM )
	return FALSE;

  if( nParams != 1 )
  {
    MSG_send(StreamNum, MSG58);		/*Nbr d'arguments incorrect*/
    Prompt;
    return TRUE;
  }

  /* Quel filtre est applique sur ce stream ? */
  nFlags = USERS_getFlags(StreamNum);
  FILTER_flags2str(nFlags, szBuffer);

  if( szBuffer[0] == SNULL )
  {
	/* Pas de filtre : indique DEVALIDE */
	MSG_get(StreamNum, MSG64, szBuffer); /* devalide */
	if( szBuffer[0] ) /* Si buffer non vide : enlever CR */
		szBuffer[strlen(szBuffer) - 1] = SNULL;
	MSG_pParams[9] = szBuffer;
  }
  else
  {
	/* Filtre actif : afficher la liste */
	MSG_pParams[9] = szBuffer;
  }

  MSG_send(StreamNum, MSG69);
  Prompt;
  return TRUE;
}


/*------------------------------------------------------------------------
  ------ SH/WWV ----------------------------------------------------------
  ------------------------------------------------------------------------*/
#pragma argsused
int CSHOW_wwv(int StreamNum, int nParams, char **pArg, char **pStrArg)
{
  char 	  szLogger[16];
  WWVSPOT WwvSpot;
  int 	  iNbr = 5;
  int	  iDone = FALSE;
  int	  index;

  /*Nombre de WWV a renvoyer*/
  if( pArg[0][3] == '/' )
  {
	if( pArg[0][4] != SNULL )	/*Proteger le atoi*/
	{
		iNbr = atoi (pArg[0] + 4);

		if( iNbr <= 0 )
			iNbr = 5;
	}
  }/*End IF*/

  /*Balayer la table des spots WWV*/
  for(index = 1; index <= iNbr; index++)
  {
    if( ! WWV_get(index, &WwvSpot) )
	  break;	/*Fin de fichier atteinte*/

    /*Ajouter un ">" a la fin de l'indicatif (affichage <CALLSIGN>)*/
    strcpy(szLogger, WwvSpot.szLogger);
    strcat(szLogger, ">");

    /*Si c'est le premier spot, envoyer un entete*/
    if( iDone == FALSE )
    {
	iDone = TRUE;
	MSG_send(StreamNum, MSG42);
    } /*End IF*/

    /*Envoyer le spot WWV a l'utilisateur*/
    BUFFERS_printBuff(StreamNum, OUT,
		      "%11s  %c%c   %3d %3d %3d  %-35s <%-6s\n",
		      WwvSpot.szDate, WwvSpot.szTime[0], WwvSpot.szTime[1],
		      WwvSpot.nSFI, WwvSpot.nA, WwvSpot.nK,
		      WwvSpot.szForecast, szLogger);
  }/*End FOR*/

  Prompt;
  return TRUE;
}
