#include <alloc.h>
#include <stdio.h>
#include <string.h>
#include "dx.h"
#include "define.h"
#include "buffers.h"
#include "symbol.h"
#include "tools.h"

extern char	STREAMS_callsign[MAX_STREAMS][10];
extern char	STREAMS_level[MAX_STREAMS];

/*-------------------------------------------------------------------------
  ------ Ajouter un DX dans le fichier DX ---------------------------------
  -------------------------------------------------------------------------*/
void DX_add(char * pszDxCall, unsigned long lFrequency, char * pszComments, char * pszTime, char * pszDate, char * pszLogger, char * pszFromPC)
{
	FILE * fPtr;
	DXSPOT DxSpot;

	/* Limiter la longueur des champs */
	TOOLS_maxLength(pszFromPC,    9);
	TOOLS_maxLength(pszLogger,    9);
	TOOLS_maxLength(pszDxCall,   14);
	TOOLS_maxLength(pszComments, 30);
	TOOLS_maxLength(pszDate,     11);
	TOOLS_maxLength(pszTime,      4);

	/* Preparer les champs de l'enregistrement */
	memset(&DxSpot, 0, sizeof(DxSpot)); /* Fichier plus clean */
	strcpy(DxSpot.szFromPC,   pszFromPC);
	strcpy(DxSpot.szLogger,   pszLogger);
	strcpy(DxSpot.szDxCall,   pszDxCall);
	strcpy(DxSpot.szComments, pszComments);
	strcpy(DxSpot.szDate,	  pszDate);
	strcpy(DxSpot.szTime,	  pszTime);
	DxSpot.lFrequency = lFrequency;

	/* Ouvrir le fichier pour placer le nouveau DX a la fin */
	fPtr = fopen(DX_FILE, "a+b");

	if( ! fPtr )	/* Erreur d'ouverture du fichier */
	{
		perror("fopen in DX_add");
		return;
	}

	/* Enregistrer le DX a la fin du fichier */
	fwrite(&DxSpot, sizeof(DXSPOT), 1, fPtr);
	fclose(fPtr);
}

/*-------------------------------------------------------------------------
  ------ Vrifier la validit de la frquence -----------------------------
  -------------------------------------------------------------------------
  La fonctionne retourne la bande correspodant  la Frquence et 0 sinon   */
int DX_checkFrequency(unsigned long lFrequency)
{
	int Done = 0;

	if( lFrequency >= 18000L 	&&	lFrequency <= 20000L )
		Done = 160;
	else
	if( lFrequency >= 35000L 	&&	lFrequency <= 40000L )
		Done = 80;
	else
	if( lFrequency >= 70000L 	&&	lFrequency <= 73000L )
		Done = 40;
	else
	if( lFrequency >= 100000L 	&&	lFrequency <= 101500L )
		Done = 30;
	else
	if( lFrequency >= 140000L 	&&	lFrequency <= 143500L )
		Done = 20;
	else
	if( lFrequency >= 180680L 	&&	lFrequency <= 181680L )
		Done = 17;
	else
	if( lFrequency >= 210000L 	&&	lFrequency <= 214500L )
		Done = 15;
	else
	if( lFrequency >= 248900L 	&&	lFrequency <= 249900L )
		Done = 12;
	else
	if( lFrequency >= 280000L 	&&	lFrequency <= 297000L )
		Done = 10;
	else
	if( lFrequency >= 500000L 	&&	lFrequency <= 540000L )
		Done = 6;
	else
	if( lFrequency >= 700000L 	&&	lFrequency <= 705000L )
		Done = 4;
	else
	if( lFrequency >= 1440000L 	&&	lFrequency <= 1480000L )
		Done = 2;
	else
	if( lFrequency >= 2200000L 	&&	lFrequency <= 2250000L )
		Done = 135;
	else
	if( lFrequency >= 4200000L 	&&	lFrequency <= 4500000L )
		Done = 70;
	else
	if( lFrequency >= 9020000L 	&&	lFrequency <= 9280000L )
		Done = 34;
	else
	if( lFrequency >= 12400000L 	&&	lFrequency <= 13000000L )
		Done = 23;
	else
	if( lFrequency >= 23000000L 	&&	lFrequency <= 24500000L )
		Done = 13;
	else
	if( lFrequency >= 56500000L 	&&	lFrequency <= 58500000L )
		Done = 5;
	else
	if( lFrequency >= 100000000L 	&&	lFrequency <= 105000000L )
		Done = 3;
	else
	if( lFrequency >= 240000000L 	&&	lFrequency <= 245000000L )
		Done = 1;

	return Done;
}
/*-------------------------------------------------------------------------
  ------ Charger un DX du fichier DX --------------------------------------
  -------------------------------------------------------------------------
  La fonction retourne TRUE si le DX a pu etre lu                          */
int DX_get(int nPos, DXSPOT * pDxSpot)
{
	FILE *fPtr;
	long  lOffset;

	/* Ouvrir le fichier */
	fPtr = fopen(DX_FILE, "rb");

	if( ! fPtr ) /* Erreur d'ouverture du fichier */
	{
		perror("fopen in DX_get");
		return FALSE;
	}

	/* Se placer  la fin du fichier */
	fseek(fPtr, 0L, SEEK_END);

	/* Remonter de nPos enregistrements */
	lOffset =  (long) nPos * (long) sizeof(DXSPOT);
	if( lOffset > ftell(fPtr) )
	{
		fclose(fPtr);
		return FALSE;
	}
	fseek(fPtr, -lOffset, SEEK_CUR);

	/* Extraire les champs */
	fread(pDxSpot, sizeof(DXSPOT), 1, fPtr);

	/* Fermer le fichier et retourner */
	fclose( fPtr );
	return TRUE;
}

/*---------------------------------------------------------------------------
  ------ Rechercher si le spot DX est un double -----------------------------
  ---------------------------------------------------------------------------
  La fonction retourne TRUE si le DX est double */
int DX_isDupe(char * pszDxCall, unsigned long lFrequency, char * pszComments, char * pszTime, char * pszLogger)
{
	#define	NLAST	50L
	FILE   *fPtr;
	long	lOffset;
	int	index;
	char	szLogger[16];
	DXSPOT  DxSpot;

	/* Recherche de DUPE a effectuer qqs le SSID du logger */
	strcpy(szLogger, pszLogger);
	TOOLS_removeSsid(szLogger);

	/* Ouvrir le fichier */
	fPtr = fopen(DX_FILE, "rb");

	if( ! fPtr )	/* Impossible d'ouvrir */
	{
		perror("fopen in DX_isDupe");
		return FALSE;
	}

	/* Placer le pointeur de fichier NLAST spots en arriere */
	fseek(fPtr, 0L, SEEK_END);			/* Fin du fichier */
	lOffset =  NLAST * (long) sizeof(DXSPOT);	/* Calcul offset */
	if( lOffset > ftell(fPtr) )
		fseek(fPtr, 0L, SEEK_SET);		/* Moins de NLAST spots enregistres : se placer en debut de fichier */
	else
		fseek(fPtr, -lOffset, SEEK_CUR);	/* Pointer NLAST */

	/* Tester chacun des spots */
	for(index = 0; index < NLAST; index++)
	{
		/* Lecture enregistrement */
		if( ! fread(&DxSpot, sizeof(DXSPOT), 1, fPtr) )
			break;	/* EOF */

		/* Comparer ... */
		TOOLS_removeSsid(DxSpot.szLogger);	/* Qqs le SSID du logger */

		if( ! strcmp(pszDxCall,   DxSpot.szDxCall)   &&
		      lFrequency    ==    DxSpot.lFrequency  &&
		    ! strcmp(pszComments, DxSpot.szComments) &&
		    ! strcmp(pszTime,     DxSpot.szTime)     &&
		    ! strcmp(szLogger,    DxSpot.szLogger)  )
		{
			/* DUPE */
			fclose( fPtr );
			return TRUE;
		}

	}/* End FOR */

	/* Fermer le fichier et retourner */
	fclose( fPtr );
	return FALSE;
}